﻿// Licence information see ArduinoXML.as

class ArduinoFO {
	var updateIntervalId:Number;
	var io_array:Array = Array(); 
	var updateInterval:Number = 40;// in ms 
	var laststring:String;// copy of the last string send
	var sendstring:String;// string to send to Arduino
	var send_error:Boolean = false; // in case of error send laststring again
	
	// variables for display values in the gui
	var gui_text_array:Array = Array();
	var gui_received_string:String = "";
	var gui_error_text:String = "";
	var gui_error_array:Array = Array();	

	//----------- Construct the arduino object.
	function ArduinoFO(updateTime:Number, pinConfig:Array) {
		
		/* The constructor will configurate the Arduino object in Flash autmaticaly based on the parameters. 
		The parameter updateTime will override the default uptdatetime of 20 ms (it's advised not to make it
		smaller). The parameter pinConfig can be used to set the pinconfiguration that is used in the Arduino.
		Based on this pinconfig the input and outputarray is formed. The outputarray will be send to Arduino
		which will use it to set the outputs. You have to make sure that the program in the Arduino sets the 
		same configuration and can handle the outputarray that Flash sends automaticaly. Output pin 0 and 1 
		cannot be used on Arduino because you want to have serial communication so these are used for Tx and Rx.
		*/

		if (updateTime != undefined) {
			updateInterval = updateTime;// use given uptdatetime instead of default
		}
		if (pinConfig != undefined) {
			io_array[0] = 0;
			gui_text_array[0] = "R";
			io_array[1] = 0;
			gui_text_array[1] = "T";

			for (var i = 0; i<pinConfig.length; i++) {
				if (pinConfig[i] == "i") {
					io_array.push(0); // add numberelement to array
					gui_text_array.push("i");
				}
				if (pinConfig[i] == "o") {
					io_array.push(0); // add numberelement to array
					gui_text_array.push("o");
				}
				if (pinConfig[i] == "p") {		
					io_array.push("000"); // add stringelement to output array
					gui_text_array.push("pwm");
				}
			}
			for (var i = 0; i<6; i++) {
				io_array.push("000");
				gui_text_array.push("an"+i);
			}
		}
	}
	
	/* ------------------- Sending information to Arduino 
	Flash will control the update frequency of Arduino. Arduino will only react on the 
	communication from Flash. The three functions below control the update frequency 
	and the information send to Arduino.
	*/
	
	public function updateArduino() {
		sendOutputArray();
	}
	public function intervalTimer() {
		updateIntervalId = setInterval(this, "updateArduino", updateInterval);
	}

	public function sendOutputArray() { 
		var j = 0; 
		var output_array:Array = Array(); // make the output_array

		// copy the outputvalues from the io_array to the output_array
		for (var i = 0; i<_root.pinConfig.length; i++) {
		
		  // make sure pwm values are formated like "001", "010";
		  if (_root.pinConfig[i] == "p") {
		    if (io_array[i+2].length<3) {			    
			  io_array[i+2] = "00"+io_array[i+2];				
			  if (io_array[i+2].length>3) io_array[i+2] = io_array[i+2].slice(1);			    
		    }			
		  }
			
			
		  if (_root.pinConfig[i] != "i") {
		    output_array[j] = io_array[i+2];
			j++;
		  }
		}
		
		// start with an o so Arduino processes the right info
		// you can make strings yourself as well
		sendstring = "o"+output_array.join(",")+","; 
		
		// check if the output settings have changed
		// if not, only an n character is send. This keeps the serial line 
		// fast ( prevending errors by too much information ). 
		
		// in case of error sendstring has to be send again
		if(send_error==true) 
		{ // send the laststring again
		  _root.axml.send(laststring); // send the string to Arduino 
		  send_error=false;
		}
		else
		{ if (sendstring == laststring) {
			_root.axml.send("n"); // send an n			
		  }
		  else
		  { laststring = sendstring; // Remember what you've send this time
			_root.axml.send(sendstring); // send the string to Arduino 
		  } 
		}	
	}
	
	/* ------------------- Listen for serial communication with Arduino 
	When Arduino sends information to Flash, needs to get active. This is done with a listener
	when Arduino sends information to the XML socket it will signal this and activate other things.
	When the connection is succesfull it will start the repeated update sequence mentioned above.
	*/

	function initializeListener() {

		_root.arduinoListener.onConnect = function(success:Boolean) {
			if (success) {
				_root.arduino.intervalTimer();// there is connection and communication will start.

			}
		};
		/*------------------------------- Serial Listener getting Inputs
		When the XML-socket receives data it will read the data and store it in the arrays.
		The data is given by the event object and first stored in a temporary local array.
		*/
		_root.arduinoListener.onReceiveData = function(evtObj:Object) {
			var receivedstring:String = evtObj.data; // store the received data in a string
			
			var receivedstring_array:Array = receivedstring.split(","); //convert string to array

			var inputtype = receivedstring_array.shift(); // put first character in variable

			if (inputtype == "e") 
			{ // set error boolean to true. sendoutputArray() will send the last string,
			  // the one that didn't arrive, again. 				
			  _root.arduino.send_error=true;					
			
			  /// display error information in GUI, remember the last three messages
			  var received_error_string="e"+receivedstring_array.shift()+","+receivedstring_array.shift();
			  var items = _root.arduino.gui_error_array.unshift(received_error_string+"\n");
			  if (items>3) _root.arduino.gui_error_array.pop();
			  _root.arduino.gui_error_text = _root.arduino.gui_error_array.join("");				  
			}
			
			// inputtype is a normal string
			if (inputtype == "i" || inputtype == "e") { 
				var j:Number = 0;

				for (var i = 0; i<_root.pinConfig.length; i++) {
					if (_root.pinConfig[i] == "i") {
						_root.arduino.io_array[i+2] = Number(receivedstring_array[j]);
						j++;
					}
				}
				// analog inputs
				for (var i = 0; i<6; i++) {
					// display the text_array in a nice way
					var characterslength = receivedstring_array[j].length;					
					if(characterslength<2)      _root.arduino.gui_text_array[i+14]=i;
					else if(characterslength<3) _root.arduino.gui_text_array[i+14]="a"+i;
					else if(characterslength<4) _root.arduino.gui_text_array[i+14]="an"+i;
					else                         _root.arduino.gui_text_array[i+14]="ana"+i;				
					
					_root.arduino.io_array[i+14] = Number(receivedstring_array[j]);							
					j++;
				}
				
				// display information in GUI
				_root.arduino.gui_received_string = inputtype+receivedstring_array.join(",");
			}
			
			
			
			
			
			
			_root.makeVisible();// call the funcion in the root change things in your movieclip.
		};
	}
}